<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 26, 2019
 */

use function PHPSTORM_META\type;

/**
 * Nodeworx Controller for Acronis integration
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Ctrl_Nodeworx_Acronis extends Ctrl_Nodeworx_Plugin
{
    const ACTIVE_TAB = 'active_tab';
    const IS_AUTHORIZED = 'authorized';
    const INSTALL_BACKUP_AGENT = 'install_backup_agent';

    const AB_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const AB_MESSAGE_API_FAILURE = '##LG_ACRONIS_API_FAILURE##';
    const AB_MESSAGE_UNKNOWN_ERROR = '##LG_UNKNOWN_ERROR##';
    const AB_MESSAGE_INSTALLATION_ERROR = '##LG_ACRONIS_INSTALLATION_ERROR##';
    const AB_MESSAGE_INSTALL_SUCCESS = '##LG_ACRONIS_INSTALL_SUCCESS##';
    const AB_MESSAGE_AGENT_ALREADY_INSTALLED = '##LG_ACRONIS_AGENT_ALREADY_INSTALLED##';
    const AB_MESSAGE_NO_BACKUP_AGENT = '##LG_ACRONIS_NO_BACKUP_AGENT##';
    const AB_MESSAGE_UNINSTALL_ERROR = '##LG_ACRONIS_UNINSTALL_ERROR##';
    const AB_MESSAGE_UNINSTALL_SUCCESS = '##LG_ACRONIS_UNINSTALL_SUCCESS##';
    const AB_MESSAGE_INVALID_PATH = '##LG_ACRONIS_INVALID_PATH##';
    const AB_MESSAGE_ALREADY_ACTIVATED = '##LG_ACRONIS_ALREADY_ACTIVE##';
    const AB_MESSAGE_NO_PLAN_ASSIGNED = '##LG_ACRONIS_NO_PLAN_ASSIGNED##';
    const AB_MESSAGE_RESUME_RECOVERY_ERROR = '##LG_ACRONIS_RESUME_RECOVERY_ERROR##';
    const AB_MESSAGE_RECOVERY_IN_PROGRESS = '##LG_RECOVERY_IN_PROGRESS##';
    const AB_MESSAGE_RECOVERY_IN_PROGRESS_ALT_LOCATION = '##LG_RECOVERY_IN_PROGRESS_LOCATION##';
    const AB_MESSAGE_DB_RECOVERY_NOT_ENABLED = '##LG_DB_RECOVERY_NOT_ENABLED_NW##';

    private $_Plugin;
    private $_ActivitiesLogger;

    /**
     * block actions if not master nodeworx user.
     *
     * @param string $action
     * @return boolean
     */
    public function isActionAllowed($action)
    {
        if (IW::NW()->isReseller() || !$this->_Plugin->isCurrentNwUserAllowed()) {
            $this->_blockAction('##LG_ACTION_NOT_PERMITTED##');
            return false;
        }

        if (Ini::get(Ini::IWORX, 'goiworx') == '0') {
            return true;
        }

        $parentCheck = parent::isActionAllowed($action);
        if ($parentCheck === true) {
            if (IW::NW()->isReseller() === true) {
                if ($this->_isAllowedAsReseller($action) === false) {
                    $this->_blockAction('##LG_ACTION_NOT_PERMITTED##');
                    return false;
                }
            }
        }
        return $parentCheck;
    }

    /**
     * Is allowed as reseller.
     *
     * @param string $action
     * @return boolean
     */
    protected function _isAllowedAsReseller($action)
    {
        return false;
    }

    /**
     * Init function. Final construct will run this method.
     */
    protected function _init()
    {
        $this->_Plugin = $this->_getPlugin();
    }


    /**
     * Index action.
     */
    public function indexAction()
    {
        // ONLY UNCOMMENT FOR DEBUG PURPOSES
        // $this->_Plugin->getCache()->deleteAll();

        $isAuthorized = $this->_Plugin->userIsAuthorized();

        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, $isAuthorized);
        $View->assign('nw_user', true);

        if (!$isAuthorized) {
            $this->_displayFailure('_notAuthorized', self::AB_MESSAGE_NOT_AUTHORIZED);
        }


        $BackupAgent = $this->_Plugin->getBackupAgent();

        $backupAgentNotInstalled = !$BackupAgent->isInstalled();
        $scriptsMissing = $BackupAgent->scriptsMissing();
        if ($isAuthorized && ($backupAgentNotInstalled || $scriptsMissing)) {
            $View->assign(self::INSTALL_BACKUP_AGENT, true);
        }
    }

    /**
     * Exists because index action can't refer to itself
     * when displayFailure is triggered.
     */
    private function _notAuthorizedAction()
    {
        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, false);
    }

    /**
     * Install backup agent.
     *
     * @param Form_SubmitOnly $Form
     */
    public function installBackupAgentCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_installBackupAgent', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _installBackupAgent(Form_SubmitOnly $Form)
    {
        $BackupAgent = $this->_Plugin->getBackupAgent();

        if ($BackupAgent->isInstalled()) {
            $BackupAgent->addPrePostScriptsIfMissing();
        }

        if (!$BackupAgent->isInstalled()) {
            $BackupAgent->install();
        }

        $this->_displaySuccess(self::AB_MESSAGE_INSTALL_SUCCESS);
    }

    /**
     * Uninstall backup agent.
     *
     * @param Form_SubmitOnly $Form
     */
    public function uninstallBackupAgentCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_uninstallBackupAgent', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _uninstallBackupAgent(Form_SubmitOnly $Form)
    {
        $BackupAgent = $this->_Plugin->getBackupAgent();
        $BackupAgent->uninstall();
        $this->_displaySuccess(self::AB_MESSAGE_UNINSTALL_SUCCESS);
    }

    /**
     * Get dashboard data.
     *
     * @param Form_SubmitOnly $Form
     */
    public function getDashboardDataCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_getDashboardData', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _getDashboardData(Form_SubmitOnly $Form)
    {

        $storageData = $this->_getStorageData();
        $planDetails =  $this->_getBackupPlanDetails();
        $backupStatus = $this->_getBackupStatusFromResource();

        $payloadData = $this->_mergeArrays(array($storageData, $planDetails, $backupStatus));
        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * merge multiple arrays into one array.
     *
     * @param array $arrayOfArrays
     * @return array
     */
    private function _mergeArrays(array $arrayOfArrays)
    {
        $result = array();

        foreach ($arrayOfArrays as $arr) {
            $result = array_merge($result, $arr);
        }

        return $result;
    }

    /**
     * get storage data.
     *
     * @return array
     */
    private function _getStorageData()
    {
        $Server = $this->_Plugin->getServer();
        return $Server->getStorageData();
    }

    /**
     * Get cloud storage usage data.
     *
     * @param Form_SubmitOnly $Form
     */
    public function getStorageSizeCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_displayStorageSize', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _displayStorageSize(Form_SubmitOnly $Form)
    {
        $storageData = $this->_getStorageData();
        $size = $storageData['size'];
        $Payload = new Payload($size);
        $this->_displayPayload($Payload);
    }

    /**
     * Get backup plan status (enabled/disabled) and backup plan schedule.
     *
     * @param Form_SubmitOnly $Form
     */
    public function getBackupPlanDetailsCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_displayBackupPlan', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    public function _displayBackupPlan(Form_SubmitOnly $Form)
    {
        $payloadData = $this->_getBackupPlanDetails();
        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * @return array
     */
    private function _getBackupPlanDetails()
    {
        $planDetails = array();

        if ($this->_isProtected()) {
            $BackupPlan = $this->_getBackupPlan();
            $planDetails = $BackupPlan->getPlanDetails();
        }

        return $planDetails;
    }

    /**
     * Get backup plan details.
     *
     * @return Model_Backups_BackupPlan
     */
    private function _getBackupPlan()
    {
        $planId = $this->_getCurrentPlanId();
        $BackupPlan = $this->_Plugin->getBackupPlan($planId);
        return $BackupPlan;
    }

    /**
     * Display last backup status, date, and time.
     *
     * @param Form_SubmitOnly $Form
     */
    public function getBackupActivitiesCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_displayBackupInfo', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _displayBackupInfo(Form_SubmitOnly $Form)
    {
        $backupInfo = $this->_getBackupStatusFromResource();
        $this->_displayPayloadAsJson($backupInfo);
    }


    /**
     * Get storage data.
     *
     * @return array
     */
    private function _getBackupStatusFromResource()
    {
        $Server = $this->_Plugin->getServer();

        // only display last backup & status if next backup is scheduled.
        $result = $this->_isProtected() ? $Server->getBackupStatus() : array();
        return $result;
    }

    /**
     * Get redirect URL for one-time login. Link is valid for 30 seconds.
     *
     * @param Form_SubmitOnly $Form
     */
    public function redirectToBackupCloudCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_redirectToBackupCloud', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _redirectToBackupCloud(Form_SubmitOnly $Form)
    {
        $Auth = $this->_Plugin->getAuth();
        $loginLink = $Auth->getOneTimeLoginLink();
        $data = array('redirect' => $loginLink);
        $this->_displayPayloadAsJson($data);
    }

    /**
     * Revoke backup plan.
     *
     * @param Form_SubmitOnly $Form
     */
    public function revokePlanCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_revokePlan', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _revokePlan(Form_SubmitOnly $Form)
    {
        if (!$this->_isProtected()) {
            throw new Exception_BackupPlan_NoPlanAssigned();
        }

        $BackupPlan = $this->_getBackupPlan();
        $this->_removeResourceFromPlan($BackupPlan);

        $this->_awaitApiUpdateRevokePlan();
        $this->_displayPayload(new Payload('Backup plan was successfully revoked.'));
    }

    /**
     * Remove resource from backup plan.
     *
     * @param Model_Backups_BackupPlan $BackupPlan
     */
    private function _removeResourceFromPlan(Model_Backups_BackupPlan $BackupPlan)
    {
        $resourceId = $this->_getResourceId();
        $BackupPlan->removeResource($resourceId);
    }

    /**
     * Await api update - should update to not protected
     * when plan is revoked.
     */
    private function _awaitApiUpdateRevokePlan()
    {
        $attempts = 0;

        while ($this->_isProtected() && $attempts < 20) {
            $attempts++;
            sleep(1);
        }
    }

    /**
     * List name and id of compatible backup plans. Does not include selected plans.
     */
    public function listCompatiblePlansAction()
    {
        $plans = $this->_getApplicablePlans();
        $this->_displayPayloadAsJson($plans);
    }

    /**
     * Activate interworx default plan. Will revoke current plan.
     *
     * @param Form_SubmitOnly $Form
     */
    public function activateInterWorxDefaultPlanCommitAction(Form_SubmitOnly $Form)
    {
        if ($this->_isProtected()) {
            $BackupPlan = $this->_getBackupPlan();
            $this->_removeResourceFromPlan($BackupPlan);
        }

        try {
            $this->_addResourceToExistingIworxPlan();
        } catch (Exception $e) {
            $this->_createNewPlan();
        }

        $this->_displayPayloadActivatePlan();
    }

    /**
     * Add resource to existing plan.q
     */
    private function _addResourceToExistingIworxPlan()
    {
        $DefaultPlan = $this->_getDefaultPlanFromExisting();
        $this->_addResourceToPlan($DefaultPlan);
    }

    /**
     * @return Model_Backups_BackupPlan
     */
    private function _getDefaultPlanFromExisting()
    {
        $plans = $this->_getApplicablePlans();

        foreach ($plans as $plan) {
            $id = $plan['ID'];
            $BackupPlan = new Model_Backups_BackupPlan($id);
            if ($BackupPlan->getName() === Model_Backups_Api::DEFAULT_NAME && $BackupPlan->prePostScriptsOK()) {
                return $BackupPlan;
            }
        }

        throw new Exception('no default plan was found in availabla plans');
    }

    /**
     * @return array
     */
    private function _getApplicablePlans()
    {
        $Server = $this->_getPlugin()->getServer();
        $plans = $Server->getApplicablePlans();
        return $plans;
    }

    /**
     * Activate backup plan.
     *
     * @param Form_AbActivatePlan $Form
     */
    public function activatePlanCommitAction(Form_AbActivatePlan $Form)
    {
        $this->_executeFunctionAndCatchException('_activatePlan', $Form);
    }

    /**
     * @param Form_AbActivatePlan $Form
     */
    private function _activatePlan(Form_AbActivatePlan $Form)
    {
        if ($this->_isProtected()) {
            $this->_replaceCurrentPlan($Form->getSelectedPlanId());
        } else {
            $this->_applyOrCreateBackupPlan($Form);
            $this->_awaitApiUpdateNextBackup();
        }

        $this->_displayPayloadActivatePlan();
    }

    /**
     * check if resource is protected by backup plan.
     *
     * @return boolean
     */
    private function _isProtected()
    {
        $Server = $this->_Plugin->getServer();
        return $Server->isProtected();
    }

    /**
     * switch backup plan.
     *
     * @param string $newPlanId
     */
    private function _replaceCurrentPlan($newPlanId)
    {
        if (!isset($newPlanId)) {
            throw new Exception_BackupPlan_AlreadyActivated('Plan already activated');
        }

        $oldPlanId = $this->_getCurrentPlanId();
        $this->_removeResourceFromPlan(new Model_Backups_BackupPlan($oldPlanId));
        $this->_addResourceToPlan(new Model_Backups_BackupPlan($newPlanId));
    }

    /**
     * @param Model_Backups_BackupPlan $BackupPlan
     */
    private function _addResourceToPlan(Model_Backups_BackupPlan $BackupPlan)
    {
        $resourceId = $this->_getResourceId();
        $BackupPlan->addResource($resourceId);
    }

    /**
     * Get current plan id.
     */
    private function _getCurrentPlanId()
    {
        return $this->_Plugin->getServer()->getPlanId();
    }

    /**
     * display successful payload after activatePlan.
     */
    private function _displayPayloadActivatePlan()
    {
        $planId = $this->_getCurrentPlanId();
        $BackupPlan = $this->_Plugin->getBackupPlan($planId);

        $payloadData = $BackupPlan->getPlanDetails();
        $payloadData['message'] = 'Backup plan successfully activated';
        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * Attempt to activate existing plan.
     * Create new plan if existing plans are invalid,
     * or if none exists.
     *
     * @param Form_AbActivatePlan $Form
     */
    private function _applyOrCreateBackupPlan(Form_AbActivatePlan $Form)
    {
        try {
            $selected = $Form->getSelectedPlanId();
            $planId = isset($selected) ? $selected : $Form->getDefaultPlanId();
            $BackupPlan = $this->_Plugin->getBackupPlan($planId);
            $this->_addResourceToPlan($BackupPlan);
        } catch (Exception $e) {
            // No compatible plans exists. Create new one.
            $this->_createNewPlan();
        }
    }

    /**
     * Create new default plan.
     */
    private function _createNewPlan()
    {
        $this->_Plugin->getNewPlan();
    }

    /**
     * @return string
     */
    private function _getResourceId()
    {
        return $this->_Plugin->getServer()->getResourceId();
    }

    /**
     * Await backup status change to 'running'.
     */
    private function _awaitApiUpdateNextBackup()
    {
        $Server = $this->_Plugin->getServer();
        $attempts = 0;

        while ($Server->backupNotScheduled() && $attempts < 10) {
            sleep(2);
            $attempts++;
        }
    }

    /**
     * Get alerts.
     *
     * @param Form_SubmitOnly $Form
     */
    public function getAlertsCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_getAlerts', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _getAlerts(Form_SubmitOnly $Form)
    {
        $AlertsManager = $this->_Plugin->getAlertManager();
        $alerts = $AlertsManager->getAlerts();
        $this->_displayPayloadAsJson($alerts);
    }

    /**
     * Run backup.
     *
     * @param Form_SubmitOnly $Form
     */
    public function runBackupCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_runBackup', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _runBackup(Form_SubmitOnly $Form)
    {
        $planId = $this->_getCurrentPlanId();
        $BackupPlan = $BackupPlan = $this->_Plugin->getBackupPlan($planId);
        $resourceId = $this->_getResourceId();

        $BackupPlan->runBackup($resourceId);
        $this->_awaitResourceBackupStatus();

        $backupInfo = $this->_getBackupStatusFromResource();
        $backupInfo['message'] = 'backup successfully started'; // TODO add to lang file
        $this->_displayPayloadAsJson($backupInfo);
    }

    /**
     * Await backup status change to 'running'.
     */
    private function _awaitResourceBackupStatus()
    {
        $Server = $this->_Plugin->getServer();
        $attempts = 0;

        while (!$Server->backupInProgress() && $attempts < 10) {
            sleep(2);
            $attempts++;
        }
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listBackupsCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_listBackups', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _listBackups(Form_SubmitOnly $Form)
    {
        $backups = array();

        if ($this->_isProtected()) {
            $BackupsRetriever = $this->_Plugin->getBackupsRetriever();
            $backups = $BackupsRetriever->getBackups();
        }

        $this->_displayPayloadAsJson($backups);
    }

    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    public function getRootContentCommitAction(Form_AcronisBackups $Form)
    {
        $this->_executeFunctionAndCatchException('_getRootContent', $Form);
    }

    /**
     * @param Form_AcronisBackups $Form
     */
    private function _getRootContent(Form_AcronisBackups $Form)
    {
        $backupId = $Form->getBackupId();
        $FilesRestorer = $this->_Plugin->getNwFilesRecoveryPoint($backupId);
        $dirContent = $FilesRestorer->getRootContent();

        $this->_displayPayloadAsJson($dirContent);
    }

    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    public function getDirectoryContentCommitAction(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $this->_executeFunctionAndCatchException('_getDirectoryContent', $Form);
    }

    /**
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    private function _getDirectoryContent(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $backupId = $Form->getBackupId();
        $dirPath = $Form->getDirectoryPath();

        $FilesRestorer = $this->_Plugin->getNwFilesRecoveryPoint($backupId);
        $dirContent = $FilesRestorer->getDirectoryContent($dirPath);

        $this->_displayPayloadAsJson($dirContent);
    }


    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    public function recoverDirectoriesCommitAction(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $this->_executeFunctionAndCatchException('_recoverDirectories', $Form);
    }

    /**
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    private function _recoverDirectories(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $backupId = $Form->getBackupId();
        $dirPaths = $Form->getDirectoryPaths();
        $shouldDeleteOlderFiles = $Form->deleteRequested();

        $FilesRestorer = $this->_Plugin->getNwFilesRecoveryPoint($backupId);
        $FilesRestorer->recoverDirectories($dirPaths, $shouldDeleteOlderFiles);
        $planId = $FilesRestorer->getPlanId();

        $this->_awaitRecoveryApiUpdate($planId);

        $result = array(
            'message' => self::AB_MESSAGE_RECOVERY_IN_PROGRESS,
            'planId' => $planId,
            'directories' => $dirPaths
        );

        $this->_displayPayloadAsJson($result);
    }

    /**
     * Await acronis api to update - including plan id.
     *
     * @param string $planId
     */
    private function _awaitRecoveryApiUpdate($planId)
    {
        $attempts = 0;
        while (!$this->_planExistsInApi($planId) && $attempts < 20) {
            $attempts++;
            sleep(3);
        }
    }

    /**
     * Check if recovery plan exists in acronis api.
     *
     * @param string $planId
     */
    private function _planExistsInApi($planId)
    {
        try {
            // will throw exception if planid not found in api
            $RecoveryPlan = new Model_Recovery_Plan($planId);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Check recovery state & progress by recovery plan id.
     *
     * @param Form_AcronisBackups_RecoveryProgress $Form
     */
    public function getRecoveryProgressAction(Form_AcronisBackups_RecoveryProgress $Form)
    {
        $this->_executeFunctionAndCatchException('_getRecoveryProgress', $Form);
    }

    /**
     * Check recovery state by recovery plan id.
     *
     * @param Form_AcronisBackups_RecoveryProgress $Form
     */
    private function _getRecoveryProgress(Form_AcronisBackups_RecoveryProgress $Form)
    {
        $planId = $Form->getPlanId();
        $RecoveryPlan = new Model_Recovery_Plan($planId);

        $result = array(
            'progress' => $RecoveryPlan->getProgress(),
            'state' => $RecoveryPlan->getState(),
            'recoveryId' => $RecoveryPlan->getUuid(),
            'planId' => $RecoveryPlan->getPlanId(),
            'recoveryError' => $RecoveryPlan->getError()
        );

        $this->_displayPayloadAsJson($result);
    }

    /**
     * List recoveries.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listRecoveriesCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_listRecoveries', $Form);
    }

    /**
     * @param Form_SubmitOnly $Form
     */
    private function _listRecoveries(Form_SubmitOnly $Form)
    {
        $RecoveryPlans = new Model_Recovery_Plans();
        $plans = $RecoveryPlans->getAll();
        $this->_displayPayloadAsJson($plans);
    }

    /**
     * Resume recovery if state is "paused".
     *
     * @param Form_AcronisBackups_ResumeRecovery $Form
     */
    public function resumeRecoveryAction(Form_AcronisBackups_ResumeRecovery $Form)
    {
        $this->_executeFunctionAndCatchException('_resumeRecovery', $Form);
    }

    /**
     * Resume recovery if state is "paused".
     *
     * @param Form_AcronisBackups_ResumeRecovery $Form
     */
    private function _resumeRecovery(Form_AcronisBackups_ResumeRecovery $Form)
    {
        $recoveryId = $Form->getRecoveryId();
        $RecoveryPlans = new Model_Recovery_Plans();
        $RecoveryPlans->resumeRecovery($recoveryId);

        $this->_displayPayloadAsJson(array(
            'message' => 'recovery resumed'
        ));
    }

    /**
     * Get restore location.
     */
    public function getRestoreAltLocationAction()
    {
        $altLocation = Model_Recovery_Files_Nodeworx::getRestoreAltLocationPath();
        $PayloadData = array(
            'altLocation' => $altLocation
        );

        $this->_displayPayloadAsJson($PayloadData);
    }

    /**
     * Recover directories to alt location.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    public function restoreToAltLocationCommitAction(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreToAltLocation', $Form);
    }

    /**
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    private function _restoreToAltLocation(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $backupId = $Form->getBackupId();
        $dirPaths = $Form->getDirectoryPaths();

        $FilesRestorer = $this->_Plugin->getNwFilesRecoveryPoint($backupId);
        $FilesRestorer->restoreToAltLocation($dirPaths);
        $planId = $FilesRestorer->getPlanId();

        $this->_getActivitiesLogger()->logDownloadFiles($dirPaths);
        $this->_awaitRecoveryApiUpdate($planId);

        $altLocation = Model_Recovery_Files_Nodeworx::getRestoreAltLocationPath();

        $result = array(
            'message' => self::AB_MESSAGE_RECOVERY_IN_PROGRESS_ALT_LOCATION . " $altLocation",
            'planId' => $planId,
            'directories' => $dirPaths
        );

        $this->_displayPayloadAsJson($result);
    }

    /**
     * List databases.
     *
     * @param Form_AcronisBackups $Form
     */
    public function listDatabasesCommitAction(Form_AcronisBackups $Form)
    {
        $this->_executeFunctionAndCatchException('_listDatabases', $Form);
    }

    /**
     * @param Form_AcronisBackups $Form
     */
    private function _listDatabases(Form_AcronisBackups $Form)
    {
        $backupId = $Form->getBackupId();
        $DbRestorer = $this->_Plugin->getNwDatabaseRecoveryPoint($backupId);
        $dbs = $DbRestorer->getDbs();
        $this->_displayPayloadAsJson($dbs);
    }

    /**
     * Restore datbases.
     *
     * @param Form_AcronisBackups_RecoverDatabases $Form
     */
    public function restoreDatabasesCommitAction(Form_AcronisBackups_RecoverDatabases $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabases', $Form);
    }

    /**
     * @param Form_AcronisBackups_RecoverDatabases $Form
     */
    private function _restoreDatabases(Form_AcronisBackups_RecoverDatabases $Form)
    {
        $unixUser = $Form->getUnixUser();
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();

        $DbRestorer = $this->_Plugin->getNwDatabaseRecoveryPoint($backupId);
        $DbRestorer->restoreDatabases($unixUser, $databases);

        $payloadData = array(
            'message' => 'Databases successfully recovered',
            'unixuser' => $unixUser,
            'databases' => $databases
        );

        $this->_getActivitiesLogger()->logRestoreDatabases($databases);
        $this->_displayPayloadAsJson($payloadData);
    }

    /**
     * Restore Databases to alt location.
     *
     * @param Form_AcronisBackups_RecoverDatabases $Form
     */
    public function restoreDatabasesAltLocationCommitAction(Form_AcronisBackups_RecoverDatabases $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabasesAltLocation', $Form);
    }

    /**
     * @param Form_AcronisBackups_RecoverDatabases $Form
     */
    private function _restoreDatabasesAltLocation(Form_AcronisBackups_RecoverDatabases $Form)
    {
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();
        $unixUser = $Form->getUnixUser();

        $DbRestorer = $this->_Plugin->getNwDatabaseRecoveryPoint($backupId);
        $DbRestorer->restoreDatabasesAltLocation($unixUser, $databases);

        $this->_getActivitiesLogger()->logDownloadDatabases($databases);

        $Payload = new Payload('Databases successfully recovered');
        $this->_displayPayload($Payload);
    }

    /**
     * @param string $functionToCall
     * @param Form   $Form
     */
    private function _executeFunctionAndCatchException($functionToCall, Form $Form)
    {
        try {
            $this->$functionToCall($Form);
        } catch (Exception_AcronisApiFailure $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        } catch (Exception_BackupAgentInstallFailure $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_INSTALLATION_ERROR);
        } catch (Exception_BackupAgentUninstallError $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_UNINSTALL_ERROR);
        } catch (Exception_InvalidCredentials $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_NOT_AUTHORIZED);
        } catch (Exception_InvalidPath $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_INVALID_PATH);
        } catch (Exception_BackupPlan_AlreadyActivated $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_ALREADY_ACTIVATED);
        } catch (Exception_BackupPlan_NoPlanAssigned $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_NO_PLAN_ASSIGNED);
        } catch (Exception_InvalidInput $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', $e->getMessage());
        } catch (Exception_ResumeRecovery $e) {
            $this->_logError($functionToCall, $e);

            $state = $e->getRecoveryState();
            $msg = self::AB_MESSAGE_RESUME_RECOVERY_ERROR . " $state";
            $this->_displayFailure('index', $msg);
        } catch (Exception_DbRecoveryNotEnabled $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_DB_RECOVERY_NOT_ENABLED);
        } catch (Exception $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_UNKNOWN_ERROR);
        }
    }

    /**
     * Log error msg.
     *
     * @param string    $functionName
     * @param Exception $e
     */
    private function _logError($functionName, Exception $e)
    {
        $exceptionDetails = $this->_getMessageFromException($e);
        IW::getLog()->error("Action failed $functionName: $exceptionDetails");
    }

    /**
     * Get message from exception
     *
     * @param Exception $e
     */
    private function _getMessageFromException(Exception $e)
    {
        $class = $e->getFile();
        $line = $e->getLine();
        $message = $e->getMessage();
        return "$message. File: $class, line: $line";
    }

    /**
     * Display json if cli
     *
     * @param mixed $payloadContent
     */
    private function _displayPayloadAsJson($payloadContent)
    {
        // render as json
        if (IW::Env()->matches(IW::CLI)) {
            IW::Env()->setOutputStyle('pretty');
        }

        $Payload = new Payload($payloadContent);
        $this->_displayPayload($Payload);
    }

    /**
     * get nw activities logger.
     *
     * @return Model_Logger_Activities_Nodeworx
     */
    protected function _getActivitiesLogger()
    {
        if (!isset($this->_ActivitiesLogger)) {
            $this->_ActivitiesLogger = new Model_Logger_Activities_Nodeworx();
        }

        return $this->_ActivitiesLogger;
    }
}
