<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 14, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
abstract class Model_Recovery_Files extends Model_Acronis
{
    const RECOVERY_DIR_NAME = 'acronis_temp';
    const NW_DOWNLOAD_DIR_PATH = '/home/' . self::RECOVERY_DIR_NAME;
    const DIR_ID = 'id';
    const DIR_CONTENT = 'content';

    private $_backupId;
    private $_legacyId;
    private $_machineId;
    private $_tenantAgentId;
    private $_created;

    /**
     * @param string $backupId
     */
    public function __construct($backupId)
    {
        $Cache = $this->_getCache();

        $this->_backupId = $backupId;
        $this->_machineId = $Cache->getMachineId();
        $this->_tenantAgentId = $Cache->getTenantAgentId();
        $this->_legacyId = $Cache->getLegacyId($backupId);
        $this->_created = $Cache->getCreatedAt($backupId);
    }

    /**
     * Get backup id.
     *
     * @return string
     */
    public function getBackupId()
    {
        return $this->_backupId;
    }

    /**
     * Get legacy id. Expects value to be cached.
     *
     * @return string
     */
    public function getLegacyId()
    {
        return $this->_legacyId;
    }

    /**
     * Get machine id. Expects value  to be cached.
     *
     * @return string
     */
    public function getMachineId()
    {
        return $this->_machineId;
    }

    /**
     * Get tenant agent id.
     *
     * @return string
     */
    public function getAgentTenantId()
    {
        return $this->_tenantAgentId;
    }

    /**
     * Get backup creation date.
     *
     * @return string
     */
    public function getCreated()
    {
        return $this->_created;
    }

    /**
     * Get subscription id.
     *
     * @return string
     */
    public function getSubscriptionId()
    {
        return $this->_getApi()->getSubscriptionId();
    }

    /**
     * Get root content.
     *
     * @return array
     */
    protected function _getRootContent()
    {
        $rootPath = '/';
        $content = $this->_getCache()->getDirContentByPath($this->_backupId, $rootPath);

        if (!isset($content)) {
            $FileLocator = new Model_Directory_FileLocator($rootPath);
            $rootDevicePath = $FileLocator->getRootDevicePath();
            $Dir = $this->_getDeviceDirectory($rootDevicePath);
            $content = $Dir->getContent($this->getLegacyId(), $this->getMachineId());

            $this->_getCache()->cacheDirContentByPath($this->_backupId, $rootPath, $content);
        }

        return $content;
    }

    /**
     * get content from /home/
     *
     * @return array
     */
    protected function _getHomeContent()
    {
        $homeContent = $this->_getDirContentByPath('/home');
        return $homeContent;
    }

    /**
     * Get content by path.
     *
     * @param string $path
     * @return array
     */
    public function _getDirContentByPath($path)
    {
        $Cache = $this->_getCache();
        $content = $Cache->getDirContentByPath($this->_backupId, $path);

        if (!isset($content)) {
            $Dir = $this->_getDirByPath($path);
            $content = $Dir->getContent($this->getLegacyId(), $this->getMachineId());
            $Cache->cacheDirContentByPath($this->_backupId, $path, $content);
        }

        return $content;
    }

    /**
     * @param string $path
     * @return Model_Directory_Dir
     */
    protected function _getDirByPath($path)
    {
        $Dir = $this->_getCachedDir($path);
        if (!isset($Dir)) {
            $Dir = $this->_getDirByPathFromApi($path);
        }

        return $Dir;
    }

    /**
     * Get cached directory.
     *
     * @param string $path
     * @return Model_Directory_Dir
     */
    private function _getCachedDir($path)
    {
        $FileLocator = new Model_Directory_FileLocator($path);
        $devicePath = $FileLocator->getFullDevicePath();
        $cachedDirDetails = $this->_getCache()->getDirByDevicePath($this->_backupId, $devicePath);

        if (!empty($cachedDirDetails)) {
            return new Model_Directory_Dir($cachedDirDetails);
        }
    }

    /**
     * @param string $path
     * @return Model_Directory_Dir
     */
    private function _getDirByPathFromApi($path)
    {
        $FileLocator = new Model_Directory_FileLocator($path);
        $deviceDirs = $FileLocator->getDeviceDirs();

        $devicePath = $FileLocator->getRootDevicePath();
        $Dir = $this->_getDeviceDirectory($devicePath);

        foreach ($deviceDirs as $subDirName) {
            $Dir = $this->_getSubDir($Dir, $subDirName);
        }

        return $Dir;
    }

    /**
     * @param Model_Directory_Dir $ParentDir
     * @param string              $subDirName
     */
    private function _getSubDir(Model_Directory_Dir $ParentDir, $subDirName)
    {
        $Dir = $this->_getCachedSubDir($ParentDir, $subDirName);

        if (!isset($dirDetails)) {
            $Dir = $ParentDir->getSubDirByName($subDirName, $this->getLegacyId(), $this->getMachineId());
            $this->_getCache()->cacheDir($this->_backupId, $Dir->getApiDetails());
        }

        return $Dir;
    }

    /**
     * Get subdir from cache.
     *
     * @param Model_Directory_Dir $ParentDir
     * @param string              $subDirName
     */
    private function _getCachedSubDir(Model_Directory_Dir $ParentDir, $subDirName)
    {
        $devicePath = $ParentDir->getPath() . "/$subDirName";
        $dirDetails = $this->_getCache()->getDirByDevicePath($this->_backupId, $devicePath);

        if (isset($dirDetails)) {
            return new Model_Directory_Dir($dirDetails);
        }
    }

    /**
     * Get device root content.
     *
     * @param string $deviceToBrowse
     * @return Model_Directory_Dir
     */
    protected function _getDeviceDirectory($deviceToBrowse)
    {
        $devices = $this->_getDirContentFromApi();
        $dirDetails = Model_Directory_Dir::getDirByName($devices, $deviceToBrowse);

        $Dir = new Model_Directory_Dir($dirDetails);
        return $Dir;
    }

    /**
     * Get dir from api.
     *
     * @param string $dirId
     * @return array
     */
    private function _getDirContentFromApi($dirId = null)
    {
        $Api = $this->_getApi();
        return $Api->getDirectoryContent($this->_legacyId, $dirId);
    }

    /**
     * @param Model_Directory_Dir[] $Dirs
     * @param boolean               $deleteNewerFiles
     */
    protected function _recoverDirectories(array $Dirs, $deleteNewerFiles = false)
    {
        if ($deleteNewerFiles) {
            $this->_deleteNewerFiles($Dirs);
        }


        $dirIds = $this->_getDirIds($Dirs);

        $Api =  $this->_getApi();
        $Api->recoverDirectories($this->_legacyId, $this->_tenantAgentId, $dirIds);
    }

    /**
     * Get plan id from api. Will only return plan id if plan has been created.
     *
     * @return string
     */
    public function getPlanId()
    {
        return $this->_getApi()->getPlanId();
    }

    /**
     * @param Model_Directory_Dir[] $Dirs
     * @return array
     */
    protected function _getDirIds(array $Dirs)
    {
        $ids = array();

        foreach ($Dirs as $path => $Dir) {
            $ids[] = $Dir->getId();
        }

        return $ids;
    }

    /**
     * Delete newer files if directory.
     *
     * @param Model_Directories_Dir[] $Dirs
     */
    protected function _deleteNewerFiles(array $Dirs)
    {
        $this->_getApi()->deleteFilesCreatedAfter($this->getCreated());

        foreach ($Dirs as $path => $Dir) {
            $this->_deleteNewContentIfDir($path, $Dir);
        }
    }

    /**
     * Delete new content if directory.
     *
     * @param string              $path
     * @param Model_Directory_Dir $Dir
     */
    private function _deleteNewContentIfDir($path, Model_Directory_Dir $Dir)
    {
        if ($Dir->getType() === 'dir') {
            $this->_getApi()->addPathDeleteAfter($path);
        }
    }

    /**
     * Get array
     *
     * @param array $paths
     */
    protected function _getDirsWithLocalPathIndex(array $paths)
    {
        $dirs = array();
        foreach ($paths as $path) {
            $path = '/' . trim($path, '/');

            $Dir = $this->_getDirByPath($path);
            $dirs[$path] = $Dir;
        }

        return $dirs;
    }

    /**
     * Create dirs for restoring to alt location & downloading content.
     *
     * @param string $path
     */
    protected function _createRestoreAltLocationDir($path)
    {
        if (!$this->_isDir($path)) {
            $mkdirCommand = "mkdir -p $path && chmod 0755 $path";
            $Command = new Command($mkdirCommand);
            $Command->runAsRoot();
            $Command->exec();
        }
    }
}
