<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 14, 2020
 */

/**
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
abstract class Model_Recovery_Files_Databases extends Model_Recovery_Files
{
    const DUMPS_DESTINATION_PATH = self::NW_DOWNLOAD_DIR_PATH . '/db-dumps';
    const DUMPS_TEMP_PATH = '/home/.acronis_tmp/pre-acronis-mysql';

    const DUMP_INFO_USER = 'unixUser';
    const DUMP_INFO_DBS = 'dbNames';

    /**
     * Save dumps info from api in cache.
     */
    protected function _cacheDumps()
    {
        $dbInfo = $this->_getDumpsInfoFromApi();
        $this->_getCache()->cacheDumps($this->getBackupId(), $dbInfo);
    }

    /**
     * Get dumps info.
     *
     * @return array
     */
    private function _getDumpsInfoFromApi()
    {
        try {
            $dumpsInfo = $this->_getApiDumpsInfo();
            return $dumpsInfo;
        } catch (Exception_InvalidPath $e) {
            throw new Exception_DbRecoveryNotEnabled();
        }
    }

    /**
     * get dumps info from api.
     * Will throw exception if dumps do not exist.
     *
     * @return array
     */
    private function _getApiDumpsInfo()
    {
        $content = $this->_getDirContentByPath(self::DUMPS_TEMP_PATH);
        $result = array_map(function ($dir) {
            $name = $dir['name'];

            if (strpos($name, '-dumps') !== false) {
                $path = self::DUMPS_TEMP_PATH . "/$name";
                return $this->_getUserDbs($path, $name);
            }
        }, $content);

        return array_values(array_filter($result));
    }

    /**
     * Get db info, user & db names.
     *
     * @param string $path
     * @param string $dirName
     */
    private function _getUserDbs($path, $dirName)
    {
        if (strpos($dirName, '-dumps') === false) {
            throw new Exception('not dbs folder: ' . $dirName);
        }

        $dbUser = str_replace('-dumps', '', $dirName);
        $dbNames = $this->_getNamesFromDirContentByPath($path);

        return array(
            self::DUMP_INFO_USER => $dbUser,
            self::DUMP_INFO_DBS => $dbNames
        );
    }

    /**
     * Excludes mysql db.
     *
     * @param string $path
     */
    private function _getNamesFromDirContentByPath($path)
    {
        $content = $this->_getDirContentByPath($path);

        $result = array_map(function ($dir) {
            if ($dir['name'] !== 'mysql') {
                return $dir['name'];
            }
        }, $content);

        return array_values(array_filter($result));
    }

    /**
     * Run command as root and get output as array.
     *
     * @param string $cmd
     * @return array
     */
    protected function _getCmdOutput($cmd)
    {
        $Command = new Command($cmd);
        $Command->runAsRoot();
        $Command->allowAsterisk();
        $Command->allowAmpersand();
        $Command->allowRedirectOut();
        $Command->exec();
        return $Command->getOutput();
    }

    /**
     * Restore user dumps.
     *
     * @param string $unixUser
     */
    protected function _restoreUserDumps($unixUser)
    {
        $legacyId = $this->getLegacyId();
        $tenantAgentId = $this->getAgentTenantId();
        $path = $this->_getDumpsPathByUser($unixUser);
        $Dir = $this->_getDirByPath($path);

        $ids = array($Dir->getId());
        $destination = self::DUMPS_DESTINATION_PATH;

        $this->_createRestoreAltLocationDir($destination);


        $this->_getApi()->recoverDirectoriesToAltLocation($legacyId, $tenantAgentId, $ids, $destination);
    }

    /**
     * Get user dumps path.
     *
     * @param string $unixUser
     */
    protected function _getDumpsPathByUser($unixUser)
    {
        return self::DUMPS_TEMP_PATH . "/$unixUser-dumps";
    }

    /**
     * Await restoration. Will wait max 3 minutes for restoration to complete.
     *
     * @param string $unixUser
     */
    protected function _awaitUserDumpsRestoration($unixUser)
    {
        $count = 1;
        while (!$this->_userDumpsRestored($unixUser) && $count < 60) {
            $count++;
            sleep(3);
        }
        if (!$this->_userDumpsRestored($unixUser)) {
            // TODO custom exception? File restoration took to long to proceed?
            throw new Exception('Dumps not restored to tmp location!');
        }
    }

    /**
     * Check if user dumps are restored.
     *
     * @param string $unixUser
     */
    protected function _userDumpsRestored($unixUser)
    {

        $path = self::DUMPS_DESTINATION_PATH . "/$unixUser-dumps";
        return $this->_isDir($path);
    }

    /**
     * unzip archives to specified destination.
     *
     * @param string $archive
     * @param string $destination
     */
    protected function _gunzip($archive, $destination)
    {
        $cmd = "gunzip -c $archive > $destination && chown iworx:iworx $destination";
        $Command = new Command($cmd);
        $Command->runAsRoot();
        $Command->allowRedirectOut();
        $Command->allowAmpersand();
        $Command->allowColon();
        $Command->exec();
    }

    /**
     * Get dumps path by unix user.
     *
     * @param string $unixUser
     */
    protected function _getDumpsDestinationPathByUnixUser($unixUser)
    {
        $dumpsPath = self::DUMPS_DESTINATION_PATH . "/$unixUser-dumps";
        return $dumpsPath;
    }

    /**
     * Get array of archives containing sql file for tables.
     *
     * @param string $parentFolder
     * @return array
     */
    protected function _getTableArchives($parentFolder)
    {
        return $this->_getCmdOutput("ls $parentFolder/*.gz 2> /dev/null");
    }

    /**
     * get sql files in specified location.
     *
     * @param string $sqlPath
     * @param string $dbName
     * @return array
     */
    protected function _getSqlFiles($sqlPath, $dbName)
    {
        $cmd = "ls $sqlPath/$dbName.*.sql 2> /dev/null";
        return $this->_getCmdOutput($cmd);
    }


    /**
     * cleanup dumps.
     */
    protected function _cleanupDumps()
    {
        $dumpsPath = self::DUMPS_DESTINATION_PATH;
        $Command = new Command("rm -rf $dumpsPath");
        $Command->runAsRoot();
        $Command->exec();
    }

    /**
     *
     * @param string $user
     * @param array  $dbNames
     * @param string $destination
     */
    protected function _restoreUserDbsToAltLocation($user, array $dbNames, $destination)
    {
        $tenantAgentId = $this->getAgentTenantId();
        $legacyId = $this->getLegacyId();
        $this->_createRestoreAltLocationDir($destination);

        $dirIds = $this->_getDbIds($user, $dbNames);
        $this->_getApi()->recoverDirectoriesToAltLocation($legacyId, $tenantAgentId, $dirIds, $destination);
    }

    /**
     * @param string $user
     * @param array  $databaseNames
     */
    private function _getDbIds($user, array $databaseNames)
    {
        $userDumpsPath = $this->_getDumpsPathByUser($user);
        $dbDirs = $this->_getDirContentByPath($userDumpsPath);

        $ids = array();
        foreach ($dbDirs as $dir) {
            if (in_array($dir['name'], $databaseNames)) {
                $ids[] = $dir['id'];
            }
        }

        return $ids;
    }
}
